'*******************************************************************************
'----------------------------- HID  USB-----------------------------
'*******************************************************************************

'BASCOM-AVR v.2.0.4.0
'Rubashka Vasiliy
'Ukraine
'2011
'$prog &HFF , &HFF , &HD9 , &HFF 'Take care that the chip supports all fuse bytes.

$noramclear
$hwstack = 40
$swstack = 40
$framesize = 50
$regfile = "m8def.dat"                                      'ATMEGA8-16PU
$crystal = 15000000                                         ' - 15

'Include the software USB library - NEW
$lib "swusb.lbx"
$external _swusb

Declare Sub Usb_reset()
Declare Sub Usb_processsetup(txstate As Byte)
Declare Sub Usb_send(txstate As Byte , Byval Count As Byte)
Declare Sub Usb_senddescriptor(txstate As Byte , Maxlen As Byte)

'***************************  USB **********************************
'
'Set the following parameters to match your hardware configuration and USB
'device parameters.

'******************************* USB Connections *******************************

'Define the AVR port that the two USB pins are connected to
_usb_port Alias Portd
_usb_pin Alias Pind
_usb_ddr Alias Ddrd

'Define the D+ and D- pins. (put D+ on an interrupt pin)
Const _usb_dplus = 2
Const _usb_dminus = 4


'          +--*--------------*---------*--------------------+
'       0,1|  |              |         |    ATMEGA 8        |
'         --- |        +-----+---[1k5]-+    +--\/--+        |
'   USB   --- |        |     +---[10k]---*--|1   28|        |
'          |  |   CAPS | +--|<|--[300]---+--|2   27|        |
'  /---\   |  |   NUM  | *--|<|--[300]---+--|3   26|        |
' | VCC|-1-+--+ +------+-+-------[68 ]---+--|4   25|        |
'  \   |   |    | LOCK | *--|<|--[300]---+--|5   24|        |
'   |D-|-2-+--*-+------*-+-------[68 ]---+--|6   23|        |
'   |  |   |  | |        |            +--+--|7   22|--------+-+
'   |D+|-3-+--+-*-L<|----*------------+--+--|8   21|        | |
'  /   |   |  |          | +----------+--+--|9   20|--------* |
' | GND|-4-*  +---L<|----* |  ===  +--+--+--|10  19|------+ | |
'  \---/   |    2x3,6v   | +-12MHz-+  |  |  |11  18|----+ | | |
'          |             |    ===     |  |  |12  17|--+ | | | |
'          +-------------*--|<|--[300]+--+--|13  16|  | | | | |
'                        |STATUS      |  |  |14  15|  | | | | |
'                        |            |  |  +------+  | | | | |
'                        |            |  +----------+ | | | | |
'                        |            *-------*-----+-+-+-+-* |
'                        |            |0,1   +|22,0 | | | | | |
'                        |           ---     ---    | | | | | |
'                        |           ---     ---    | | | | | |
'                        |            |       |     | | | | | |
'                        *------------*-------*-----+-+-+-+-+-*
'                        |                          | | | | | |
'                       ===                         V V V V V V
'
'                                                   R M M S V G
'                                                   E O I C C N
'                                                   S S S K C D

'Configure the pins as inputs

Ddrb = &B11100000                                           '   B: 1 -  
Portb = &B00011111                                          '   B: 00000000
Ddrc = &B11111111                                           '   C: 1 -  
Portc = &B00000000                                          '   C: 00000000
Ddrd = &B11101011                                           '   D: 1 -  
Portd = &B00000000                                          '   D: 00000000

'disable pullups
_usb_port._usb_dplus = 0
_usb_port._usb_dminus = 0

'************************* USB   **************************

' EEPROM  FLASH   USB 
'1 = EEPROM, 0 = FLASH.   EEPROM    .
Const _usb_use_eeprom = 0

'Don't wait for sent packets to be ACK'd by the host before marking the
'transmission as complete.  This option breaks the USB spec but improves
'throughput with faster polling speeds.
'This may cause reliability issues.  Should leave set to 0 to be safe.
Const _usb_assume_ack = 0

'*****************************  ****************************

'USB Vendor ID and Product ID (Assigned by USB-IF)
Const _usb_vid = &HBBBB
Const _usb_pid = &HEF33

'USB Device Release Number (BCD)
Const _usb_devrel = &H0001

'USB Release Spec (BCD)
Const _usb_spec = &H0110

'USB Device Class, subclass, and protocol (assigned by USB-IF).
'&h00 = Class defined by interface. (HID is defined in the interface)
'&hFF = Vendor-defined class (You must write your own PC driver)
'See http://www.usb.org/developers/defined_class  for more information
Const _usb_devclass = 0
Const _usb_devsubclass = 0
Const _usb_devprot = 0

'These are _indexes_ to UNICODE string descriptors for the manufacturer,
'product name, and serial number.  0 means there is no descriptor.
Const _usb_imanufacturer = 1
Const _usb_iproduct = 2
Const _usb_iserial = 0

'    .     ,
'   ,   !    1
Const _usb_numconfigs = 1

'*****************************   *************************

'     ( 1)
Const _usb_numifaces = 1

'  ( !)
Const _usb_confignum = 1

'   ,     (0 - )
Const _usb_iconfig = 2

'&H80 =     USB.
'&HC0 =     .
Const _usb_powered = &H80                                   '&HC0

'   * 2mA  (500mA max)
Const _usb_maxpower = 50                                    '50 * 2mA = 100mA

'****************************   ***************************

'     (1  2)
Const _usb_ifaces = 1

' 
Const _usb_ifaceaddr = 0

' 
Const _usb_alternate = 0

'      (  )
Const _usb_ifaceendpoints = 2

'USB  , ,   ( USB-IF).
'&h00 = RESERVED
'&hFF = Vendor-defined class (You must write your own PC driver)
'       Other values are USB interface device class. (such as HID)
'See http://www.usb.org/developers/defined_class  for more information
Const _usb_ifclass = 3
Const _usb_ifsubclass = 0
Const _usb_ifprotocol = 0

'        (0 - )
Const _usb_iiface = 0

'***************************  HID  *********************
'HID ,     , , .
'See http://www.usb.org/developers/hidpage/  for the specification,
'tools, and resources.

',    HID,  , ,  
'  0.     3 (HID).
'      0    
'  ,    .
'  8  9  HID 1.11 PDF.

' HID   (EXCLUDING report and physical)
'     HID,      0
Const _usb_hids = 1

'BCD HID releasenumber.  Current spec is 1.11
Const _usb_hid_release = &H0111

'Country code from page 23 of the HID 1.11 specifications.
'Usually the country code is 0 unless you are making a keyboard.
Const _usb_hid_country = 0

'        HID
'    1!  HID      1  .
Const _usb_hid_numdescriptors = 1

'Use a software tool to create the report descriptor and $INCLUDE it.


'***************************    *************************

'  0   .   -   
' .
': HID   1    

'    (  > 0.   ,   )
Const _usb_endp2addr = 1
Const _usb_endp3addr = 2

'  - 0    3  
Const _usb_endp2type = 3
Const _usb_endp3type = 3

': 0=Out ( ), 1=In ( ).    
Const _usb_endp2direction = 1
Const _usb_endp3direction = 0

'   ()    .
'    
' (    10)
Const _usb_endp2interval = 10
Const _usb_endp3interval = 10

'*******************************************************************************
'The includes need to go right here--between the configuration constants above
'and the start of the program below.  The includes will automatically calculate
'constants based on the above configuration, dimension required variables, and
'allocate transmit and receive buffers.  Nothing inside the includes file needs
'to be modified.
$include "Const_swusb-includes.bas"
'*******************************************************************************

'**************************** USB Interrupt And Init ***************************
'  , ,   
Call Usb_reset()
Const _usb_intf = Intf0
Config Int0 = Rising
On Int0 Usb_isr Nosave
Enable Int0

Enable Interrupts                                           '  

'***************************   USB ***********************
'*******************************************************************************

Dim Resetcounter As Word
Dim Idlemode As Byte
Dim Flaginputmsg As Byte                                    '  
Dim Keymsg(8) As Byte                                       '  (8 )
Dim Key(35) As Byte                                         '  
Dim Lastkey(35) As Byte                                     '   
Dim K As Byte                                               '   
Dim Paus As Byte                                            '   

Paus = 30
Portc = 1
Portb.5 = 1

' 
Config Timer1 = Timer , Prescale = 256
On Timer1 Setflag
Enable Timer1

'   -  
Portd.7 = 1
Waitms 100
Portd.7 = 0
Waitms 300
Portd.7 = 1
Waitms 100
Portd.7 = 0
Waitms 300
Portd.7 = 1
Waitms 100
Portd.7 = 0

'     - usb_tx_buffer:

   _usb_tx_buffer2(2) = 0                                   ' modifier    -   8-  (   Ctrl, Alt, Shift, Win)
   _usb_tx_buffer2(3) = 0                                   ' reserved    - 
   _usb_tx_buffer2(4) = 0                                   ' keycode 0 -   0
   _usb_tx_buffer2(5) = 0                                   ' keycode 1 -   1
   _usb_tx_buffer2(6) = 0                                   ' keycode 2 -   2
   _usb_tx_buffer2(7) = 0                                   ' keycode 3 -   3
   _usb_tx_buffer2(8) = 0                                   ' keycode 4 -   4
   _usb_tx_buffer2(9) = 0                                   ' keycode 5 -   5

' modifier
' bit0  LEFT CTRL   (1)
' bit1  LEFT SHIFT  (2)
' bit2  LEFT ALT    (4)
' bit3  LEFT WIN    (8)
' bit4  RIGHT CTRL  (16)
' bit5  RIGHT SHIFT (32)
' bit6  RIGHT ALT   (64)
' bit7  RIGHT WIN   (128)

' CTRL + ALT + DEL
'     : &H05, &H00, &H4, &H00, &H00, &H00, &H00, &H00 = 5[Ctrl(1) + Alt(4)], 0, 76, 0, 0, 0, 0, 0
'        (   Ctrl, Alt, Shift, Win)

'*******************************   *********************************
Do

   Resetcounter = 0

   ' 
   While _usb_pin._usb_dminus = 0
      Incr Resetcounter
      If Resetcounter = 1000 Then
         Call Usb_reset()
      End If
   Wend

   '  
   If _usb_status._usb_rxc = 1 Then

      If _usb_status._usb_setup = 1 Then
         ' /  
         Call Usb_processsetup(_usb_tx_status)
      Elseif _usb_status._usb_endp1 = 1 Then                '     


             Portd.0 = _usb_rx_buffer(2).0                  '   CAPS
             Portd.1 = _usb_rx_buffer(2).1                  '   NUM
             Portd.3 = _usb_rx_buffer(2).2                  '   LOCK

      End If

      '  RXC    RTR (   )
      _usb_status._usb_rtr = 1
      _usb_status._usb_rxc = 0
   End If


  If Flaginputmsg = 1 Then

'***************************   ******************************

   '             1 Col 2 Col 3 Col 4 Col 5 Col 6 Col 7 Col
   '              PC5   PC4   PC3   PC2   PC1   PC0   PB5
   '               |     |     |     |     |     |     |
   '               V     V     V     V     V     V     V
   '               |     |     |     |     |     |     |
   '1 line PB4 <---0-----0-----0-----0-----0-----0-----0---
   '               |     |     |     |     |     |     |
   '2 line PB3 <---0-----0-----0-----0-----0-----0-----0---
   '               |     |     |     |     |     |     |
   '3 line PB2 <---0-----0-----0-----0-----0-----0-----0---
   '               |     |     |     |     |     |     |
   '4 line PB1 <---0-----0-----0-----0-----0-----0-----0---
   '               |     |     |     |     |     |     |
   '5 line PB0 <---0-----0-----0-----0-----0-----0-----0---
   '               |     |     |     |     |     |     |

   Portc.5 = 0 : Key(1) = Pinb.4 : Key(8) = Pinb.3 : Key(15) = Pinb.2 : Key(22) = Pinb.1 : Key(29) = Pinb.0 : Portc.5 = 1       '
   Portc.4 = 0 : Key(2) = Pinb.4 : Key(9) = Pinb.3 : Key(16) = Pinb.2 : Key(23) = Pinb.1 : Key(30) = Pinb.0 : Portc.4 = 1       '
   Portc.3 = 0 : Key(3) = Pinb.4 : Key(10) = Pinb.3 : Key(17) = Pinb.2 : Key(24) = Pinb.1 : Key(31) = Pinb.0 : Portc.3 = 1       '
   Portc.2 = 0 : Key(4) = Pinb.4 : Key(11) = Pinb.3 : Key(18) = Pinb.2 : Key(25) = Pinb.1 : Key(32) = Pinb.0 : Portc.2 = 1       '
   Portc.1 = 0 : Key(5) = Pinb.4 : Key(12) = Pinb.3 : Key(19) = Pinb.2 : Key(26) = Pinb.1 : Key(33) = Pinb.0 : Portc.1 = 1       '
   Portc.0 = 0 : Key(6) = Pinb.4 : Key(13) = Pinb.3 : Key(20) = Pinb.2 : Key(27) = Pinb.1 : Key(34) = Pinb.0 : Portc.0 = 1       '
   Portb.5 = 0 : Key(7) = Pinb.4 : Key(14) = Pinb.3 : Key(21) = Pinb.2 : Key(28) = Pinb.1 : Key(35) = Pinb.0 : Portb.5 = 1       '


'*************************     ****************************
   For K = 1 To 33                                          '    33 
   If Key(k) <> Lastkey(k) Then                             '   
    Lastkey(k) = Key(k)
     Flaginputmsg = 0
    If Key(k) = 0 Then
      Keymsg(3) = 3 + K                                     '     7
      Gosub Send_u8                                         '  
    Else
      Keymsg(3) = 0
      Gosub Send_u8                                         '  
    End If
   End If
   Next K
  End If

'************************    ****************************
If Key(34) = 0 Then                                         ' CTRL + ALT + DEL
          Portd.7 = 1
     If _usb_tx_status._usb_txc = 1 Then
             _usb_tx_buffer2(2) = 5                         '  
             _usb_tx_buffer2(4) = 76
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          '  
            _usb_tx_buffer2(4) = 0
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
         Portd.7 = 0
 End If

'**********************     **************************

  If Key(35) = 0 Then
          Portd.7 = 1
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 2                          'Q
            _usb_tx_buffer2(4) = 20
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          'w
            _usb_tx_buffer2(4) = 26
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          'e
            _usb_tx_buffer2(4) = 8
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          'r
            _usb_tx_buffer2(4) = 21
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          't
            _usb_tx_buffer2(4) = 23
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          'y
            _usb_tx_buffer2(4) = 28
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 2                          '!
            _usb_tx_buffer2(4) = 30
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          ' 
            _usb_tx_buffer2(4) = 44
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If
     If _usb_tx_status._usb_txc = 1 Then
            _usb_tx_buffer2(2) = 0                          '  
            _usb_tx_buffer2(4) = 0
           Call Usb_send(_usb_tx_status2 , 8)               '   
           Waitms Paus
     End If

         Portd.7 = 0
  End If
Loop
'***************************    *****************************

'************************     ***************************
Send_u8:

  If Flaginputmsg = 0 Then
         If _usb_tx_status._usb_txc = 1 Then
         Portd.7 = 1
            _usb_tx_buffer2(4) = Keymsg(3)
           Call Usb_send(_usb_tx_status2 , 8)               '   
         Portd.7 = 0
         End If
   End If
Return


End

'*******************************************************************************
'**************** ,   EEPROM  FLASH ******************
'                        !
'
#if _usb_use_eeprom = 1
   $eeprom
#else
   $data
#endif

' 
_usb_devicedescriptor:
Data 18 , 18 , _usb_desc_device , _usb_specl , _usb_spech , _usb_devclass
Data _usb_devsubclass , _usb_devprot , 8 , _usb_vidl , _usb_vidh , _usb_pidl
Data _usb_pidh , _usb_devrell , _usb_devrelh , _usb_imanufacturer
Data _usb_iproduct , _usb_iserial , _usb_numconfigs


'Retrieving the configuration descriptor also gets all the interface and
'endpoint descriptors for that configuration.  It is not possible to retrieve
'only an interface or only an endpoint descriptor.  Consequently, this is a
'large transaction of variable size.

_usb_configdescriptor:
Data _usb_descr_total , 9 , _usb_desc_config , _usb_descr_totall
Data _usb_descr_totalh , _usb_numifaces , _usb_confignum , _usb_iconfig
Data _usb_powered , _usb_maxpower

'_usb_IFaceDescriptor
Data 9 , _usb_desc_iface , _usb_ifaceaddr , _usb_alternate
Data _usb_ifaceendpoints , _usb_ifclass , _usb_ifsubclass , _usb_ifprotocol
Data _usb_iiface

#if _usb_hids > 0
'_usb_HIDDescriptor
Data _usb_hid_descr_len , _usb_desc_hid , _usb_hid_releasel , _usb_hid_releaseh
Data _usb_hid_country , _usb_hid_numdescriptors

'Next follows a list of bType and wLength bytes/words for each report and
'physical descriptor.  There must be at least 1 report descriptor.  In practice,
'There are usually 0 physical descriptors and only 1 report descriptor.

Data _usb_desc_report
Data 63 , 0                                                 '      ,     HID - _usb_hid_reportdescriptor
'End of report/physical descriptor list
#endif

#if _usb_endpoints > 1
'_usb_EndpointDescriptor
Data 7 , _usb_desc_endpoint , _usb_endp2attr , _usb_endp2type , 8 , 0
Data _usb_endp2interval
#endif

#if _usb_endpoints > 2
'_usb_EndpointDescriptor
Data 7 , _usb_desc_endpoint , _usb_endp3attr , _usb_endp3type , 8 , 0
Data _usb_endp3interval
#endif

#if _usb_hids > 0
_usb_hid_reportdescriptor:
' HID-
'Data 59                                                     '   HID,     _usb_desc_report
'Data &H05 , &H01                                            ' Usage Page (Generic Desktop)
'Data &H09 , &H06                                            ' Usage (Keyboard)
'Data &HA1 , &H01                                            ' Collection (Application)
'Data &H05 , &H07                                            ' Usage Page (Keyboard)
'Data &H19 , 224                                             ' Usage Minimum (224)
'Data &H29 , 231                                             ' Usage Maximum (231)
'Data &H15 , &H00                                            ' Logical Minimum (0)
'Data &H25 , &H01                                            ' Logical Maximum (1)
'Data &H75 , &H01                                            ' Report Size (1)
'Data &H95 , &H08                                            ' Report Count (8)
'Data &H81 , &H02                                            ' Input (Data, Variable, Absolute)
'Data &H81 , &H01                                            ' Input (Constant)
'Data &H19 , &H00                                            ' Usage Minimum (0)
'Data &H29 , 101                                             ' Usage Maximum (101)
'Data &H15 , &H00                                            ' Logical Minimum (0)
'Data &H25 , 101                                             ' Logical Maximum (101)
'Data &H75 , &H08                                            ' Report Size (8)
'Data &H95 , &H06                                            ' Report Count (6)
'Data &H81 , &H00                                            ' Input (Data, Array)
'Data &H05 , &H08                                            ' Usage Page (LED)
'Data &H19 , &H01                                            ' Usage Minimum (1)
'Data &H29 , &H05                                            ' Usage Maximum (5)
'Data &H15 , &H00                                            ' Logical Minimum (0)
'Data &H25 , &H01                                            ' Logical Maximum (1)
'Data &H75 , &H01                                            ' Report Size (1)
'Data &H95 , &H05                                            ' Report Count (5)
'Data &H91 , &H02                                            ' Output (Data, Variable, Absolute)
'Data &H95 , &H03                                            ' Report Count(3)
'Data &H91 , &H01                                            ' Output(constant)
'Data &HC0                                                   ' End Collection

Data 63                                                     '   HID,     _usb_desc_report
Data &H05 , &H01                                            '   USAGE_PAGE (Generic Desktop)
Data &H09 , &H06                                            '   USAGE (Keyboard)
Data &HA1 , &H01                                            '   COLLECTION (Application)
Data &H05 , &H07                                            '   USAGE_PAGE (Keyboard)
Data &H19 , &HE0                                            '   USAGE_MINIMUM (Keyboard LeftControl)
Data &H29 , &HE7                                            '   USAGE_MAXIMUM (Keyboard Right GUI)
Data &H15 , &H00                                            '   LOGICAL_MINIMUM (0)
Data &H25 , &H01                                            '   LOGICAL_MAXIMUM (1)
Data &H75 , &H01                                            '   REPORT_SIZE (1)
Data &H95 , &H08                                            '   REPORT_COUNT (8)
Data &H81 , &H02                                            '   INPUT (Data,Var,Abs) ** Modifier Byte **
Data &H95 , &H01                                            '   REPORT_COUNT (1)
Data &H75 , &H08                                            '   REPORT_SIZE (8)
Data &H81 , &H03                                            '   INPUT (Cnst,Var,Abs) ** Reserved Byte **
Data &H95 , &H05                                            '   REPORT_COUNT (5)
Data &H75 , &H01                                            '   REPORT_SIZE (1)
Data &H05 , &H08                                            '   USAGE_PAGE (LEDs)
Data &H19 , &H01                                            '   USAGE_MINIMUM (Num Lock)
Data &H29 , &H05                                            '   USAGE_MAXIMUM (Kana)
Data &H91 , &H02                                            '   OUTPUT (Data,Var,Abs) ** LED Report **
Data &H95 , &H01                                            '   REPORT_COUNT (1)
Data &H75 , &H03                                            '   REPORT_SIZE (3)
Data &H91 , &H03                                            '   OUTPUT (Cnst,Var,Abs) ** LED Report Padding **
Data &H95 , &H06                                            '   REPORT_COUNT (6) ** here we define the maximum number of simultaneous keystrokes we can detect **
Data &H75 , &H08                                            '   REPORT_SIZE (8)
Data &H15 , &H00                                            '   LOGICAL_MINIMUM (0)
Data &H25 , &H65                                            '   LOGICAL_MAXIMUM (101)
Data &H05 , &H07                                            '   USAGE_PAGE (Keyboard)
Data &H19 , &H00                                            '   USAGE_MINIMUM (Reserved (no event indicated))
Data &H29 , &H65                                            '   USAGE_MAXIMUM (Keyboard Application)
Data &H81 , &H00                                            '   INPUT (Data,Ary,Abs) ** Key arrays (6 bytes) **
Data &HC0                                                   '   END_COLLECTION


#endif

'*****************************String descriptors********************************
' Yes, they MUST be written like "t","e","s","t".  Doing so pads them with
' 0's.  If you write it like "test," I promise you it won't work.
'   : "t","e","s","t".
'   "test," -   

'Default language descriptor (index 0)
'    ( 0)
_usb_langdescriptor:
Data 4 , 4 , _usb_desc_string , 09 , 04                     '&h0409 = English
'Data 4 , 4 , _usb_desc_string , 19 , 04                     '&h0419 = 

'Manufacturer Descriptor (unicode)
'  (unicode)
_usb_mandescriptor:
Data 14 , 14 , _usb_desc_string
Data "R" , "a" , "d" , "a" , "n" , 0 , 0

'Product Descriptor (unicode)
'  (unicode)
_usb_proddescriptor:
Data 24 , 24 , _usb_desc_string
Data "K" , "L" , "A" , "V" , "A" , " " , "A" , "T" , "O" , "M" , 0 , 0


' 
_usb_numdescriptor:
Data 6 , 6 , _usb_desc_string
Data "1" , 0 , 0

'*******************************************************************************
'************************* Subroutines -  ***************************
'*******************************************************************************

Sub Usb_processsetup(txstate As Byte)
Senddescriptor = 0

   'Control transfers reset the sync bits like so
   Txstate = _usb_setup_sync

   'These are the standard device, interface, and endpoint requests that the
   'USB spec requires that we support.
   Select Case _usb_rx_buffer(2)
      '  
      Case &B10000000:
         Select Case _usb_rx_buffer(3)
           ' Case _usb_req_get_status:                       ' 00 -   

            Case _usb_req_get_descriptor:
               Select Case _usb_rx_buffer(5)
                  Case _usb_desc_device:
                     '  
                     #if _usb_use_eeprom = 1
                        Readeeprom _usb_eepromaddrl , _usb_devicedescriptor
                     #else
                        Restore _usb_devicedescriptor
                     #endif
                     Senddescriptor = 1
                  Case _usb_desc_config:
                     '  
                     #if _usb_use_eeprom = 1
                        Readeeprom _usb_eepromaddrl , _usb_configdescriptor
                     #else
                        Restore _usb_configdescriptor
                     #endif
                     Senddescriptor = 1
                  Case _usb_desc_string:
                     Select Case _usb_rx_buffer(4)
                        Case 0:
                           '   
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_langdescriptor
                           #else
                              Restore _usb_langdescriptor
                           #endif
                           Senddescriptor = 1
                        Case 1:
                           '  
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_mandescriptor
                           #else
                              Restore _usb_mandescriptor
                           #endif
                           Senddescriptor = 1
                        Case 2:
                           '  
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_proddescriptor
                           #else
                              Restore _usb_proddescriptor
                           #endif
                           Senddescriptor = 1
                        Case 3:
                           '  
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_numdescriptor
                           #else
                              Restore _usb_numdescriptor
                           #endif
                           Senddescriptor = 1

                     End Select
               End Select
'            CASE _usb_REQ_GET_CONFIG:
         End Select
      Case &B00000000:
         Select Case _usb_rx_buffer(3)
'            CASE _usb_REQ_CLEAR_FEATURE: ' 01  
'            CASE _usb_REQ_SET_FEATURE:   ' 03  
            Case _usb_req_set_address:
               'USB status reporting for control writes
               Call Usb_send(txstate , 0)
               While Txstate._usb_txc = 0 : Wend
               'We are now addressed.
               _usb_deviceid = _usb_rx_buffer(4)
'            CASE _usb_REQ_SET_DESCRIPTOR:
            Case _usb_req_set_config:
               'Have to do status reporting
               Call Usb_send(txstate , 0)
         End Select
      '  
      Case &B10000001:
         Select Case _usb_rx_buffer(3)
'            CASE _usb_REQ_GET_STATUS:
'            CASE _usb_REQ_GET_IFACE:
            Case _usb_req_get_descriptor
            '_usb_rx_buffer(4)    (5)  
               Select Case _usb_rx_buffer(5)
                  Case _usb_desc_report:
                     #if _usb_use_eeprom = 1
                        Readeeprom _usb_eepromaddrl , _usb_hid_reportdescriptor
                     #else
                        Restore _usb_hid_reportdescriptor
                     #endif
                     Senddescriptor = 1

'                  CASE _usb_DESC_PHYSICAL

'                  CASE _USB_DESC_HID

               End Select
         End Select
      Case &B00100001:
         'Class specific SET requests
         Select Case _usb_rx_buffer(3)
            'CASE _usb_REQ_SET_REPORT:
            Case _usb_req_set_idle:
               Idlemode = 1
               'Do status reporting
               Call Usb_send(txstate , 0)
            'CASE _usb_REQ_SET_PROTOCOL:
         End Select
   End Select

   If Senddescriptor = 1 Then
      Call Usb_senddescriptor(txstate , _usb_rx_buffer(8))
   End If

End Sub

Sub Usb_senddescriptor(txstate As Byte , Maxlen As Byte)

   'Break the descriptor into packets and send to TxState
   Local Size As Byte
   Local I As Byte
   Local J As Byte
   Local Timeout As Word

   #if _usb_use_eeprom = 1
      'EEPROM access is a little funky.  The size has already been fetched
      'and stored in _usb_EEPROMADDRL, and the address of the descriptor
      'is now in the EEAR register pair.

      Size = _usb_eepromaddrl

      'Fetch the location of the descriptor and use it as an address pointer
      push R24
      in R24, EEARL
      sts {_USB_EEPROMADDRL}, R24
      in R24, eearH
      sts {_USB_EEPROMADDRH}, R24
      pop R24

   #else
      Read Size
   #endif


   If Maxlen < Size Then Size = Maxlen

   I = 2
   For J = 1 To Size
      Incr I
      #if _usb_use_eeprom = 1
         Incr _usb_eepromaddr
         Readeeprom Txstate(i) , _usb_eepromaddr
      #else
         Read Txstate(i)
      #endif

      If I = 10 Or J = Size Then
         I = I - 2
         Call Usb_send(txstate , I)
         While Txstate._usb_txc = 0
            Timeout = 0
            'To prevent an infinite loop, check for reset here
            While _usb_pin._usb_dminus = 0
               Incr Timeout
               If Timeout = 1000 Then                       '
                  Call Usb_reset()
                  Exit Sub
               End If
            Wend
         Wend
         I = 2
      End If
   Next
End Sub

Sub Usb_send(txstate As Byte , Byval Count As Byte)

   'Calculates and adds the CRC16,adds the DATAx PID,
   'and signals to the ISR that the data is ready to be sent.
   '
   '"Count" is the DATA payload size.  Range is 0 to 8. Do not exceed 8!

   'Reset all the flags except TxSync and RxSync
   Txstate = Txstate And _usb_syncmask
   'Calculate the 16-bit CRC
   _usb_crc = 0
   If Count <> 0 Then
      _usb_crc = Crc16uni(txstate(3) , Count , &HFFFF , &H8005 , 1 , 1)
      Toggle _usb_crc
   End If
   'Bytes to transmit will be PID + DATA payload + CRC16
   Count = Count + 3
   Txstate = Txstate + Count
   Txstate(count) = Low(_usb_crc)
   Incr Count
   Txstate(count) = High(_usb_crc)
   'Add the appropriate DATAx PID
   Txstate(2) = _usb_pid_data1
   If Txstate._usb_txsync = 0 Then
      Txstate(2) = _usb_pid_data0
   End If
   'The last step is to signal that the packet is Ready To Transmit
   Txstate._usb_rtt = 1
   Txstate._usb_txc = 0
End Sub

Sub Usb_reset()
   'Reset the receive flags
   _usb_status._usb_rtr = 1
   _usb_status._usb_rxc = 0
   'Reset the transmit flags
   _usb_tx_status = _usb_endp_init
   #if Varexist( "_usb_Endp2Addr")
   _usb_tx_status2 = _usb_endp_init
   #endif
   #if Varexist( "_usb_Endp3Addr")
   _usb_tx_status3 = _usb_endp_init
   #endif
   'Reset the device ID to 0
   _usb_deviceid = 0
   Idlemode = 0
End Sub

'********************     1 ***********************
Setflag:
'   12,5 ?!
Timer1 = 60000
  Flaginputmsg = 1
Return